package ch.zhaw.projekt2.turnierverwaltung;

import ch.zhaw.projekt2.turnierverwaltung.main.gameScheduleView.GameController;
import ch.zhaw.projekt2.turnierverwaltung.main.gameScheduleView.GameDecorator;
import javafx.geometry.Pos;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Pane;
import javafx.scene.layout.VBox;
import javafx.scene.shape.Line;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

/**
 * Factory Decorator Class giving additional functionality to the factory and holding the listeners in itself.
 */
public class FactoryDecorator implements IsObservable {
    private Factory factory;
    private FileIO fileIO;
    private Pane pane;
    private final List<IsObserver> listener = new ArrayList<>();

    private static final Logger logger = Logger.getLogger(FactoryDecorator.class.getCanonicalName());

    /**
     * Setup of Factory Decorator with needed classes
     *
     * @param fileIO  for Reading and Saving Files
     * @param factory factory which this class adds additional functionality to
     * @param pane    standard pane
     */
    public FactoryDecorator(FileIO fileIO, Factory factory, Pane pane) {
        logger.fine("Setting up the Factory decorator");
        setFactory(factory);
        setFileIO(fileIO);
        setPane(pane);
    }

    /**
     * Setting the File IO Class for its functionality
     *
     * @param fileIO for reading and saving to and from a file
     */
    public void setFileIO(FileIO fileIO) {
        logger.finer("Setting the FileIO to the FactoryDecorator");
        this.fileIO = fileIO;
    }

    /**
     * Setting the main class
     *
     * @param factory needed for the main functionality
     */
    public void setFactory(Factory factory) {
        logger.finer("Setting the factory to the FactoryDecorator");
        this.factory = factory;
    }

    /**
     * Setting of the given pane to the factory decorator
     *
     * @param pane that will be set
     */
    public void setPane(Pane pane) {
        logger.finer("Setting the pane to the FactoryDecorator");
        this.pane = pane;
    }

    /**
     * Method adds a new Listener to the listener list
     *
     * @param observer that is being added to the Listener List
     */
    @Override
    public void addListener(IsObserver observer) {
        logger.fine("Adding Listener: " + observer);
        listener.add(observer);
    }

    /**
     * Removes a Listener from the Listener List
     *
     * @param observer the Listener to be removed
     */
    @Override
    public void removeListener(IsObserver observer) {
        logger.fine("Removing Listener: " + observer);
        listener.remove(observer);
    }

    /**
     * Opens a tournament File in connection with the File IO Class, shows error Message if error occurs while opening
     * it.
     *
     * @param tournamentFile the File to be opened
     */
    public void openTournament(FileIO.TournamentFile tournamentFile) {
        logger.finer("Trying to open tournament file:" + tournamentFile);
        try {
            factory.setTournament(fileIO.loadTournament(tournamentFile));
            openScheduleView();
            clearMessage(false);
            logger.fine("Opened tournament file successfully");
        } catch (IOException | ClassNotFoundException e) {
            e.printStackTrace();
            logger.warning("Failed to open tournament file Error: " + e);
            printMessageToFooter("Fehler beim lesen der Datei.", true);
            String msg = factory.getLanguageConfigurator().getSelectedLanguage("IOException");
            logger.warning("Failed to open tournament file Error: " + e);
            printMessageToFooter(msg, true);
        }
    }

    /**
     * Initializes the display of the Tournament list view
     */
    public void openTournamentList() {
        logger.fine("Showing TournamentList view");
        factory.showTournamentList((BorderPane) pane);
        informListener();
    }

    /**
     * Initializes the view of the participant form
     */
    public void openParticipantForm() {
        logger.fine("Showing participant form view");
        factory.showParticipantFormular((BorderPane) pane);
        informListener();
    }

    /**
     * Initializes the view of the places form
     */
    public void openPlacesForm() {
        logger.fine("Showing places form view");
        factory.showPlacesFormular((BorderPane) pane);
        informListener();
    }

    /**
     * Initializes the view of gameSchedule
     */
    public void openScheduleView() {
        factory.showGameScheduler((BorderPane) pane);
        informListener();
    }

    /**
     * Method to load all game views to show.
     * @param hBoxCenter the box where the games should be shown.
     * @param tournamentDecorator the tournamentDecorator to communicate to tournament
     * @param treeView true if the games should be arranged like a tree.
     */
    public void loadGameList(HBox hBoxCenter, TournamentDecorator tournamentDecorator, boolean treeView) {
        hBoxCenter.getChildren().clear();

        if (tournamentDecorator.getTournament() == null) {
            return;
        }

        List<List<Game>> gameList = tournamentDecorator.getTournament().getGameList();

        List<GameDecorator> gameDecoratorsList = new ArrayList<>();
        double gameBoxHeight = 0;
        double spacingFactor = 0;
        for (int i = 0; i < gameList.size(); i++) {
            List<GameDecorator> newGameDecoratorsList = new ArrayList<>();
            VBox vBox = new VBox();
            if (treeView) {
                vBox.setAlignment(Pos.CENTER);
                vBox.setSpacing(gameBoxHeight * spacingFactor);
            } else {
                vBox.setAlignment(Pos.TOP_CENTER);
            }
            spacingFactor = spacingFactor * 2 + 1;
            for (int j = 0; j < gameList.get(i).size(); j++) {
                GameDecorator gameDecorator = new GameDecorator(gameList.get(i).get(j));
                newGameDecoratorsList.add(gameDecorator);
                GameController gameController = openGameView(vBox, gameDecorator);
                if (i > 0) {
                    gameController.addListener(gameDecoratorsList.get(j * 2));
                    gameController.addListener(gameDecoratorsList.get(j * 2 + 1));
                } else if (gameBoxHeight == 0) {
                    gameBoxHeight = gameController.getGameBoxHeight();
                }
                gameDecorator.addListener(new IsObserver() {
                    @Override
                    public void update() {
                        tournamentDecorator.saveTournament();
                    }
                });
                gameController.loadContent();
            }
            hBoxCenter.getChildren().add(vBox);
            if (treeView) {
                if (i + 1 < gameList.size())
                    hBoxCenter.getChildren().add(drawLines(vBox, gameBoxHeight, 30));
            }
            gameDecoratorsList = newGameDecoratorsList;
        }
    }

    /**
     * Method to draw the lines between the game views in the tree view.
     * @param gameVBox the box with the games where lines should be drawn.
     * @param gameBoxHeight the heigth of a single game box.
     * @param lineLength the length of the horizontal lines.
     * @return a box which contains the drawn lines.
     */
    public VBox drawLines(VBox gameVBox, double gameBoxHeight, double lineLength) {
        VBox completeLineVBox = new VBox();
        completeLineVBox.setAlignment(Pos.CENTER_LEFT);
        double lineSpacing = gameVBox.getSpacing() + gameBoxHeight - 1;
        completeLineVBox.setSpacing(lineSpacing);
        for (int i = 0; i < gameVBox.getChildren().size(); i += 2) {
            HBox lineBox = new HBox();
            lineBox.setAlignment(Pos.CENTER);

            //add Lines from left Game to center
            VBox vBox = new VBox();
            vBox.setSpacing(lineSpacing);
            vBox.getChildren().add(new Line(0, 0, lineLength, 0));
            vBox.getChildren().add(new Line(0, 0, lineLength, 0));
            lineBox.getChildren().add(vBox);


            //add vertical line
            lineBox.getChildren().add(new Line(0, 0, 0, lineSpacing));

            //add line to right game
            lineBox.getChildren().add(new Line(0, 0, lineLength, 0));

            completeLineVBox.getChildren().add(lineBox);


        }
        return completeLineVBox;
    }

    /**
     * Method Initializes the Game View, in order to do that a vbox is needed and the gameDecorator
     * @param vBox used for display
     * @param gameDecorator the gameDecorator Object to communicate with game
     * @return the controller of the loaded game view.
     */
    public GameController openGameView(VBox vBox, GameDecorator gameDecorator) {

        return factory.loadGameView(vBox, gameDecorator, this);
    }

    /**
     * Method that initializes a new Message to be written on the footer, if it's an error boolean can be set.
     *
     * @param msg The message to be written
     * @param error true if an error false if not
     */
    public void printMessageToFooter(String msg, boolean error) {
        logger.fine("Initializes to write message: " + msg + " on the footer");
        factory.printMessageToFooter((BorderPane) pane, msg, error);
    }

    /**
     * Method used to clear Messages shown on the footer
     * @param error true if an error false if not
     */
    public void clearMessage(boolean error) {
        logger.fine("Initializing to clear messages from the footer");
        factory.resetFooter((BorderPane) pane, error);
    }

    /**
     * getter Method of languageConfigurator
     * @return the languageConfigurator object.
     */
    public LanguageConfigurator getLanguageConfigurator() {
        return factory.getLanguageConfigurator();
    }

    /**
     * Method that informs all listeners of an update.
     */
    public void informListener() {

        for (IsObserver observer : listener) {
            observer.update();
        }
    }
}
